@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -23,16 +23,21 @@ import static org.testng.Assert.assertNotNull;
 import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertTrue;
 import com.google.common.collect.Sets;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 import java.util.UUID;
+import java.util.stream.Collectors;
 import org.apache.pulsar.broker.PulsarServerException;
+import org.apache.pulsar.broker.service.persistent.PersistentTopic;
 import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.SubscriptionType;
 import org.apache.pulsar.common.naming.TopicName;
+import org.apache.pulsar.common.policies.data.AutoSubscriptionCreationOverride;
 import org.apache.pulsar.common.policies.data.DispatchRate;
 import org.apache.pulsar.common.policies.data.DelayedDeliveryPolicies;
 import org.apache.pulsar.common.policies.data.InactiveTopicDeleteMode;
@@ -58,14 +63,8 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
     @Override
     @BeforeClass(alwaysRun = true, timeOut = 300000)
     public void setup() throws Exception {
-        config1.setSystemTopicEnabled(true);
         config1.setDefaultNumberOfNamespaceBundles(1);
-        config1.setTopicLevelPoliciesEnabled(true);
-        config2.setSystemTopicEnabled(true);
-        config2.setTopicLevelPoliciesEnabled(true);
         config2.setDefaultNumberOfNamespaceBundles(1);
-        config3.setSystemTopicEnabled(true);
-        config3.setTopicLevelPoliciesEnabled(true);
         config3.setDefaultNumberOfNamespaceBundles(1);
         super.setup();
     }
@@ -704,7 +703,7 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
         init(namespace, persistentTopicName);
         OffloadPoliciesImpl offloadPolicies =
                 OffloadPoliciesImpl.create("s3", "region", "bucket", "endpoint", null, null, null, null,
-                8, 9, 10L, null, OffloadedReadPriority.BOOKKEEPER_FIRST);
+                8, 9, 10L, 10L, null, OffloadedReadPriority.BOOKKEEPER_FIRST);
         // local
         try {
             admin1.topicPolicies().setOffloadPolicies(persistentTopicName, offloadPolicies);
@@ -737,6 +736,61 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
                 assertNull(admin3.topicPolicies(true).getOffloadPolicies(persistentTopicName)));
     }
 
+    @Test
+    public void testRemoveReplicationClusters() throws Exception {
+        final String namespace = "pulsar/partitionedNs-" + UUID.randomUUID();
+        final String persistentTopicName = "persistent://" + namespace + "/topic" + UUID.randomUUID();
+
+        init(namespace, persistentTopicName);
+
+        // set topic replica cluster policy as `r1 r2`
+        admin1.topics().setReplicationClusters(persistentTopicName, Arrays.asList("r1", "r2"));
+        PersistentTopic topicRef =
+            (PersistentTopic) pulsar1.getBrokerService().getTopicReference(persistentTopicName + "-partition-0").get();
+        assertNotNull(topicRef);
+
+        Awaitility.await().untilAsserted(() -> {
+            List<String> replicaClusters = topicRef.getReplicators().keys().stream().sorted().collect(Collectors.toList());
+            assertEquals(replicaClusters.size(), 1);
+            assertEquals(replicaClusters.toString(), "[r2]");
+        });
+
+        // removing topic replica cluster policy, so namespace policy should take effect
+        admin1.topics().removeReplicationClusters(persistentTopicName);
+        Awaitility.await().untilAsserted(() -> {
+            List<String> replicaClusters = topicRef.getReplicators().keys().stream().sorted().collect(Collectors.toList());
+            assertEquals(replicaClusters.size(), 2);
+            assertEquals(replicaClusters.toString(), "[r2, r3]");
+        });
+    }
+
+    @Test
+    public void testReplicateAutoSubscriptionCreation() throws Exception {
+        final String namespace = "pulsar/partitionedNs-" + UUID.randomUUID();
+        final String topic = "persistent://" + namespace + "/topic" + UUID.randomUUID();
+        init(namespace, topic);
+
+        AutoSubscriptionCreationOverride autoSubscriptionCreationOverride
+                = AutoSubscriptionCreationOverride.builder().allowAutoSubscriptionCreation(true).build();
+        // local
+        admin1.topicPolicies().setAutoSubscriptionCreation(topic, autoSubscriptionCreationOverride);
+        Awaitility.await().untilAsserted(() ->
+                assertNull(admin2.topicPolicies().getAutoSubscriptionCreation(topic, false)));
+        Awaitility.await().untilAsserted(() ->
+                assertNull(admin3.topicPolicies().getAutoSubscriptionCreation(topic, false)));
+        // global
+        admin1.topicPolicies(true).setAutoSubscriptionCreation(topic, autoSubscriptionCreationOverride);
+        Awaitility.await().ignoreExceptions().untilAsserted(() -> assertEquals(admin2.topicPolicies(true)
+                .getAutoSubscriptionCreation(topic, false).isAllowAutoSubscriptionCreation(), true));
+        Awaitility.await().ignoreExceptions().untilAsserted(() -> assertEquals(admin3.topicPolicies(true)
+                .getAutoSubscriptionCreation(topic, false).isAllowAutoSubscriptionCreation(), true));
+        // remove auto subscription creation for a topic
+        admin1.topicPolicies(true).removeAutoSubscriptionCreation(topic);
+        Awaitility.await().untilAsserted(() ->
+                assertNull(admin2.topicPolicies(true).getAutoSubscriptionCreation(topic, false)));
+        Awaitility.await().untilAsserted(() ->
+                assertNull(admin3.topicPolicies(true).getAutoSubscriptionCreation(topic, false)));
+    }
 
     private void init(String namespace, String topic)
             throws PulsarAdminException, PulsarClientException, PulsarServerException {
