@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -16,7 +16,6 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-
 package org.apache.pulsar.io.kinesis;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
@@ -34,6 +33,20 @@ public class KinesisSinkConfig extends BaseKinesisConfig implements Serializable
 
     private static final long serialVersionUID = 1L;
 
+    @FieldDoc(
+            required = false,
+            defaultValue = "",
+            help = "Kinesis end-point port. It can be found at https://docs.aws.amazon.com/general/latest/gr/rande.html"
+    )
+    private Integer awsEndpointPort;
+
+    @FieldDoc(
+            required = false,
+            defaultValue = "false",
+            help = "Tell to Kinesis Client to skip certificate validation. This is useful while performing local tests, it's recommended to always validate certificates in production environments."
+    )
+    private Boolean skipCertificateValidation = false;
+
     @FieldDoc(
         required = false,
         defaultValue = "ONLY_RAW_PAYLOAD",
@@ -54,23 +67,38 @@ public class KinesisSinkConfig extends BaseKinesisConfig implements Serializable
             + "  #\n"
             + "  #   Kinesis sink creates a flatbuffer serialized paylaod with pulsar message payload, \n"
             + "  #   properties and encryptionCtx, and publishes flatbuffer payload into the configured kinesis stream."
+            + "  #\n"
+            + "  # - FULL_MESSAGE_IN_JSON_EXPAND_VALUE \n"
+            + "  #\n"
+            + "  #   Kinesis sink sends a JSON structure containing the record topic name, key, payload, properties and event time.\n"
+            + "  #   The record schema is used to convert the value to JSON."
     )
     private MessageFormat messageFormat = MessageFormat.ONLY_RAW_PAYLOAD; // default : ONLY_RAW_PAYLOAD
 
     @FieldDoc(
-        required = false,
+            defaultValue = "true",
+            help = "Value that indicates that only properties with non-null values are to be included when using "
+                + "MessageFormat.FULL_MESSAGE_IN_JSON_EXPAND_VALUE."
+    )
+    private boolean jsonIncludeNonNulls = true;
+
+    @FieldDoc(
+            defaultValue = "false",
+            help = "When set to true and the message format is FULL_MESSAGE_IN_JSON_EXPAND_VALUE the output JSON will be flattened."
+    )
+    private boolean jsonFlatten = false;
+
+    @FieldDoc(
         defaultValue = "false",
         help = "A flag to tell Pulsar IO to retain ordering when moving messages from Pulsar to Kinesis")
     private boolean retainOrdering = false;
 
     @FieldDoc(
-            required = false,
             defaultValue = "100",
             help = "The initial delay(in milliseconds) between retries.")
     private long retryInitialDelayInMillis = 100;
 
     @FieldDoc(
-            required = false,
             defaultValue = "60000",
             help = "The maximum delay(in milliseconds) between retries.")
     private long retryMaxDelayInMillis = 60000;
@@ -100,7 +128,21 @@ public class KinesisSinkConfig extends BaseKinesisConfig implements Serializable
         /**
          * Kinesis sink sends message serialized in flat-buffer.
          */
-        FULL_MESSAGE_IN_FB;
+        FULL_MESSAGE_IN_FB,
+        /**
+         * Kinesis sink sends a JSON structure containing the record topic name, key, payload, properties and event time.
+         * The record schema is used to convert the value to JSON.
+         *
+         * Example for primitive schema:
+         * {"topicName":"my-topic","key":"message-key","payload":"message-value","properties":{"prop-key":"prop-value"},"eventTime":1648502845803}
+         *
+         * Example for AVRO or JSON schema:
+         * {"topicName":"my-topic","key":"message-key","payload":{"c":"1","d":1,"e":{"a":"a"}},"properties":{"prop-key":"prop-value"},"eventTime":1648502845803}
+         *
+         * Example for KeyValue schema:
+         * {"topicName":"my-topic","key":"message-key","payload":{"value":{"c":"1","d":1,"e":{"a":"a"}},"key":{"a":"1","b":1}},"properties":{"prop-key":"prop-value"},"eventTime":1648502845803}
+         */
+        FULL_MESSAGE_IN_JSON_EXPAND_VALUE
     }
 
 }
